-- Copyright (C) 2012 Jared Krinke.
-- 
-- This is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

local elementDuplicators = {
    image = function(element)
        return Element.Image.new(element.image, element.x, element.y, element.z, element.width, element.height, element.angle, element.color)
    end,

    text = function(element)
        local content = element.text

        if isObject(element.buffer) then
            content = element.buffer.text
        end

        -- TODO: To make this completely generic, handle the non-default font case

        return Element.Text.new(content, element.x, element.y, element.z, element.characterWidth, element.characterHeight, element.angle, element.color, element.alignment)
    end,
}

ScriptedEntity = { }

function ScriptedEntity.new(elements, steps, repeatScript, ended)
    local entity = Entity.new()

    -- Add elements to the entity
    if isTable(elements) then
        local i = 1

        while i <= #elements do
            entity.elements:add(elements[i])
            i = i + 1
        end
    elseif isObject(elements) then
        elements:forEach(function(element)
            entity.elements:add(elementDuplicators[element.type](element))
        end)
    end

    entity.steps = steps
    entity.repeatScript = repeatScript
    entity.ended = ended

    entity.timer = 0
    entity.stepIndex = 1

    -- TODO: can this reuse the code below?
    entity.x = steps[1][2]
    entity.y = steps[1][3]
    entity.z = steps[1][4]
    entity.width = steps[1][5]
    entity.height = steps[1][6]
    entity.angle = steps[1][7]
    entity.color = steps[1][8]

    function entity.update(self, ms)
        local done = false

        self.timer = self.timer + ms

        if self.timer >= self.steps[self.stepIndex][1] then
            self.timer = self.timer - self.steps[self.stepIndex][1]
            self.stepIndex = self.stepIndex + 1

            self.initialX = self.x
            self.initialY = self.y
            self.initialZ = self.z
            self.initialWidth = self.width
            self.initialHeight = self.height
            self.initialAngle = self.angle
            self.initialRed = self.color.red
            self.initialGreen = self.color.green
            self.initialBlue = self.color.blue
            self.initialOpacity = self.color.opacity

            if self.stepIndex > #self.steps then
                if isBoolean(self.repeatScript) and self.repeatScript then
                    self.stepIndex = 2
                else
                    self.update = nil
                    done = true

                    if isFunction(self.ended) then
                        self:ended()
                    end
                end
            end
        end

        if not done then
            local step = self.steps[self.stepIndex]

            self.x = self.initialX + (step[2] - self.initialX) / step[1] * self.timer
            self.y = self.initialY + (step[3] - self.initialY) / step[1] * self.timer
            self.z = self.initialZ + (step[4] - self.initialZ) / step[1] * self.timer
            self.width = self.initialWidth + (step[5] - self.initialWidth) / step[1] * self.timer
            self.height = self.initialHeight + (step[6] - self.initialHeight) / step[1] * self.timer
            self.angle = self.initialAngle + (step[7] - self.initialAngle) / step[1] * self.timer
            self.color.red = self.initialRed + (step[8].red - self.initialRed) / step[1] * self.timer
            self.color.green = self.initialGreen + (step[8].green - self.initialGreen) / step[1] * self.timer
            self.color.blue = self.initialBlue + (step[8].blue - self.initialBlue) / step[1] * self.timer
            self.color.opacity = self.initialOpacity + (step[8].opacity - self.initialOpacity) / step[1] * self.timer
        else
            local step = self.steps[#self.steps]

            self.x = step[2]
            self.y = step[3]
            self.z = step[4]
            self.width = step[5]
            self.height = step[6]
            self.angle = step[7]
            self.color.red = step[8].red
            self.color.green = step[8].green
            self.color.blue = step[8].blue
            self.color.opacity = step[8].opacity
        end
    end

    return entity
end

