-- Copyright (C) 2012 Jared Krinke.
-- 
-- This is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("UI/UI.lua")
-- TODO: These settings files should be robust to broken settings files and use defaults
require("Settings/AudioSettings.lua")

-- Ensure that audio settings exist
-- TODO: Also define default settings somewhere--not like this
if not isTable(audioSettings) then
    audioSettings = { }
end

if not isBoolean(audioSettings.enabled) then
    audioSettings.enabled = true
end

if not isNumber(audioSettings.volume) then
    audioSettings.volume = 1
end

-- Apply existing audio settings
if audioSettings.enabled then
    Audio.setVolume(audioSettings.volume)
end

AudioMenu = { }

function AudioMenu.new()
    local volumes = { "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10" }
    local enabledChoices = { "Disabled", "Enabled" }
    local audioForm = Form.newGrid(1)

    function audioForm.saveSettings(self)
        File.createDirectory("Settings")

        local file = File.openWrite("Settings/AudioSettings.lua")

        file:write("audioSettings = {")
        if audioSettings.enabled then
            file:write("    enabled = true,")
        else
            file:write("    enabled = false,")
        end
        file:write("    volume = " .. audioSettings.volume .. ",")
        file:write("}")
        file:close()
    end

    function audioForm.refresh(self)
        local volume = Audio.getVolume()

        if volume > 0 then
            self.enabledChoice:select(enabledChoices[2], true)
        else
            self.enabledChoice:select(enabledChoices[1], true)
        end

        self.volumeChoice:select("" .. floor(volume * 10), true)
    end

    audioForm.enabledChoice = Choice.new("Audio", enabledChoices, function(self, choice)
        audioSettings.enabled = (choice == enabledChoices[2])

        if audioSettings.enabled then
            if audioSettings.volume <= 0 then
                audioSettings.volume = 0.1
            end
                
            Audio.setVolume(audioSettings.volume)
        else
            Audio.setVolume(0)
        end

        -- Save
        audioForm:saveSettings()
        audioForm:refresh()
    end, true)

    audioForm.volumeChoice = Choice.new("Volume", volumes, function(self, choice)
        audioSettings.volume = choice / 10
        Audio.setVolume(audioSettings.volume)

        -- Save
        audioForm:saveSettings()
        audioForm:refresh()
    end, true)

    audioForm:add(Title.new("Audio Options"))
    audioForm:add(Form.newNestedLeftFlow(1,
        Separator.new(),
        audioForm.enabledChoice,
        audioForm.volumeChoice,
        Separator.new(),
        Button.new("Back", function() Layers.pop() end)))

    local audioMenu = FormLayer.new(audioForm)

    function audioMenu.formShown(self)
        audioForm:refresh()
    end

    return audioMenu
end

