-- Copyright (C) 2012 Jared Krinke.
-- 
-- This is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("UI/UI.lua")
require("Menus/HighScore.lua")
require("Utility/MiscSettings.lua")

local maxNameLength = 17

HighScoreEntryMenu = { }

function HighScoreEntryMenu.new()
    local highScoreEntryForm = Form.newGrid(1)

    function highScoreEntryForm.submit(self, name)
        if not isString(name) or #name <= 0 then
            name = "???"
        end

        HighScore.insert(HighScore.new(name, self.score))
        HighScore.save()

        Layers.pop()
        self.entered()
    end

    highScoreEntryForm.nameInputBox = TextInputBox.new(function(self)
        MiscSettings.set("string", "name", self:getText())
        MiscSettings.save()
        highScoreEntryForm:submit(self:getText())
    end, MiscSettings.get("string", "name"), maxNameLength)

    -- UI
    highScoreEntryForm:add(Title.new("New High Score"))

    -- TODO: Text wrapping
    highScoreEntryForm:add(Form.newNestedLeftFlow(1,
        Separator.new(),
        Label.new("Congratulations, you made it"),
        Label.new("on to the high scores list!"),
        Separator.new(),
        Label.new("Enter your name:"),
        highScoreEntryForm.nameInputBox,
        Separator.new(),
        Button.new("Done", function() highScoreEntryForm:submit(highScoreEntryForm.nameInputBox:getText()) end)))

    local highScoreEntryMenu = FormLayer.new(highScoreEntryForm)

    function highScoreEntryMenu.show(self, score, entered)
        highScoreEntryForm.score = score
        highScoreEntryForm.entered = entered

        highScoreEntryForm:focus(highScoreEntryForm.nameInputBox)
        Layers.push(self)
    end

    return highScoreEntryMenu
end


