-- Copyright (C) 2012 Jared Krinke.
-- 
-- This is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("UI/UI.lua")
require("Menus/HighScore.lua")

HighScoreScreen = { }

function HighScoreScreen.new(firstLevel, levels)
    local highScoreForm = Form.newGrid(1, -200, 200, 400)

    highScoreForm.firstLevel = firstLevel
    highScoreForm.levels = levels

    -- Activate/cancel exit
    local title = Title.new("High Scores")

    title.getActive = nil

    function title.inputReceived(self, event, pressed)
        if pressed then
            if event == "activate" then
                Layers.pop()
            end
        end
    end

    -- Create form
    highScoreForm:add(title)
    highScoreForm:add(Separator.new())

    -- Add all entries
    highScoreForm.nameLabels = { }
    highScoreForm.scoreLabels = { }

    local j = 1

    highScores:forEach(function(highScore)
        local nameLabel = Label.new("")
        local scoreLabel = Label.new("", "right")

        highScoreForm:add(Form.newNestedGrid(2, nameLabel, scoreLabel))
        highScoreForm.nameLabels[j] = nameLabel
        highScoreForm.scoreLabels[j] = scoreLabel
        j = j + 1
    end)

    function highScoreForm.refresh(self)
        -- Add all scores
        local j = 1

        highScores:forEach(function(highScore)
            highScoreForm.nameLabels[j]:setText(highScore.name)
            -- TODO: String parameters should accept numbers (but note that this converts the type of the object from the C API!)
            highScoreForm.scoreLabels[j]:setText("" .. highScore.score)
            j = j + 1
        end)
    end

    local highScoreScreen = FormLayer.new(highScoreForm)

    function highScoreScreen.formShown(self)
        highScoreForm:refresh()
    end

    return highScoreScreen
end

HighScoreMenu = { }

function HighScoreMenu.new()
    local screen = HighScoreScreen.new(i, levelsPerScreen)
    
    return { show = function() screen:show() end }
end

