-- Copyright (C) 2012 Jared Krinke.
-- 
-- This is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("UI/UI.lua")
require("Menus/HighScoreMenu.lua")
require("Menus/HighScoreEntryMenu.lua")
require("Menus/InstructionsMenu.lua")
require("Menus/OptionsMenu.lua")

MainMenu = { }

function MainMenu.new()
    local mainForm = Form.newGrid(1, -200, 150, 400)

    mainForm.highScoreMenu = HighScoreMenu.new()
    mainForm.highScoreEntryMenu = HighScoreEntryMenu.new()
    mainForm.instructionsMenu = InstructionsMenu.new()
    mainForm.optionsMenu = OptionsMenu.new()

    function mainForm.startNewGame(self)
        self.gameLayer:reset()
        Layers.push(self.gameLayer)

        local showInstructions = MiscSettings.get("boolean", "showInstructions", true)

        MiscSettings.set("boolean", "showInstructions", false)
        MiscSettings.save()

        if showInstructions then
            mainForm.instructionsMenu:show()
        end
    end

    function mainForm.showHighScores(self)
        self.highScoreMenu:show()
    end

    function mainForm.showHighScoreEntry(self, score, entered)
        self.highScoreEntryMenu:show(score, entered)
    end
    
    function mainForm.exit(self)
        exit()
    end

    -- Add game layer
    mainForm.gameLayer = GameLayer.new(function(score)
        if score > HighScore.getLowestScore() then
            mainForm:showHighScoreEntry(score, function() mainForm:showHighScores() end)
        else
            mainForm:showHighScores()
        end
    end)

    -- Form
    mainForm:add(Title.new(CretanHubris.title))
    mainForm:add(Form.newNestedLeftFlow(1,
        Separator.new(),
        Button.new("Start New Game", function() mainForm:startNewGame() end, true),
        Button.new("High Scores", function() mainForm:showHighScores() end),
        Separator.new(),
        Button.new("How to Play", function() mainForm.instructionsMenu:show()  end),
        Button.new("Options", function() mainForm.optionsMenu:show()  end),
        Separator.new(),
        Button.new("Quit", function() mainForm:exit() end)))

    -- Create layer
    local mainMenu = FormLayer.new(mainForm)
    
    -- Error handling
    function mainMenu.errorOccurred(self, message)
        print(message)
    end

    return mainMenu
end

