-- Copyright (C) 2012 Jared Krinke.
-- 
-- This is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("UI/Label.lua")

local defaultColor = Color.new(1, 1, 1)
local focusedColor = Color.new(1, 0, 0)
local disabledColor = Color.new(0.6, 0.6, 0.6)

Button = {
    defaultColor = defaultColor,
    focusedColor = focusedColor,
    disabledColor = disabledColor,
}

function Button.new(text, activated, silent)
    local button = Label.new(text)

    button.active = true
    button.silent = silent

    if isFunction(activated) then
        button.activated = function()
            activated()

            local silent = button.silent
        end
    end

    button.textElement.color = defaultColor

    function button.setActive(self, active)
        local color = defaultColor

        if not active then
            color = disabledColor
        end

        self.active = active
        self:setColor(color)
    end

    function button.getActive(self)
        return self.active
    end

    function button.focused(self)
        self:setColor(focusedColor)
    end
    
    function button.unfocused(self)
        self:setColor(defaultColor)
    end

    return button
end

