-- Copyright (C) 2012 Jared Krinke.
-- 
-- This is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("UI/Label.lua")
require("UI/Button.lua")
require("UI/Form.lua")

local arrowColor = Button.disabledColor

Choice = { }

function Choice.new(text, choices, choiceChanged, skipCallbackArg)
    local label = Button.new(text .. ": ")
    local leftArrow = Label.new("< ")
    local item = Label.new("")
    local rightArrow = Label.new(" >")
    local child = Form.newNestedFlow(3, leftArrow, item, rightArrow)
    local choice = Form.newNestedBox({ left = label, center = child })
    local i = 1

    choice.child = child
    choice.choices = choices
    choice.choiceChanged = choiceChanged
    choice.choice = choice.choices[1]
    choice.label = label
    choice.item = item
    leftArrow:setColor(arrowColor)
    rightArrow:setColor(arrowColor)

    local skipCallback = false

    if isBoolean(skipCallbackArg) then
        skipCallback = true
    end

    function choice.setIndex(self, index, skipCallback)
        self.item:setText(self.choices[index])

        if index == 1 then
            leftArrow:setText("  ")
        elseif self.index == 1 then
            leftArrow:setText("< ")
        end

        if index == #self.choices then
            rightArrow:setText("  ")
        elseif self.index == #self.choices then
            rightArrow:setText(" >")
        end

        -- Re-layout the nested form containing the updated text
        self.child:layout()
        self.index = index
        self.choice = self.choices[self.index]

        if not isBoolean(skipCallback) or not skipCallback then
            if isFunction(self.choiceChanged) then
                self:choiceChanged(self.choice)
            end

            Audio.play("Sounds/Move.wav", 0.5)
        end
    end

    function choice.select(self, choice, skipCallback)
        local i = 1

        while i <= #self.choices do
            if self.choices[i] == choice then
                self:setIndex(i, skipCallback)
                break
            end

            i = i + 1
        end
    end

    function choice.getChoice(self)
        return self.choice
    end

    function choice.movedLeft(self)
        if self.index > 1 then
            self:setIndex(self.index - 1)
        end

        return true
    end

    function choice.movedRight(self)
        if self.index < #self.choices then
            self:setIndex(self.index + 1)
        end

        return true
    end

    function choice.activated(self)
        self:setIndex((self.index % #self.choices) + 1)
    end

    function choice.setActive(self, active)
        self.label:setActive(active)
    end

    function choice.getActive(self)
        return self.label:getActive()
    end

    -- Treat this form as a single component
    choice.mouseMoved = nil

    -- Set the index, but don't call the callback
    choice:setIndex(1, skipCallback)

    return choice
end

