-- Copyright (C) 2012 Jared Krinke.
-- 
-- This is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("UI/Form.lua")

local cursorSize = 24

Cursor = { }

function Cursor.new()
    local cursor = Entity.new(0, 0, 1, cursorSize, cursorSize)

    cursor.elements:add(Element.Image.new("Images/Cursor.png", 0.5, -0.5, 0, 1, 1))

    function cursor.setPosition(self, x, y)
        self.x = x
        self.y = y
    end

    return cursor
end

FormLayer = { }

function FormLayer.new(form, period, showCursor)
    -- Allow period to be overridden
    local effectivePeriod = -1

    if isNumber(period) then
        effectivePeriod = period
    end

    local formLayer = Layer.new(effectivePeriod)

    formLayer.form = form
    formLayer.showCursor = true

    if isBoolean(showCursor) then
        formLayer.showCursor = showCursor
    end

    -- Create a cusor, if necessary
    if formLayer.showCursor then
        formLayer.cursor = formLayer:addChild(Cursor.new())
    end

    function formLayer.keyPressed(self, key, pressed, character)
        return self.form:keyPressed(key, pressed, character)
    end

    function formLayer.mouseMoved(self, x, y)
        if self.showCursor then
            self.cursor:setPosition(x, y)
        end

        return self.form:mouseMoved(x, y)
    end

    function formLayer.inputReceived(self, event, pressed)
        local handled = false

        -- TODO: This shouldn't necessarily do this... instead propagate a Boolean value indicating if the event was used or not
        if event == "cancel" and pressed then
            Layers.pop()
            handled = true

            local silent = self.silent
        else
            handled = self.form:inputReceived(event, pressed)
        end

        return handled
    end

    -- TODO: Is this a good idea?
    function formLayer.errorOccurred(self, message)
        print(message)

        if not isNil(console) then
            console:open()
        end
    end

    function formLayer.show(self)
        Layers.push(self)
    end

    function formLayer.shown(self)
        -- Adjust cursor, if necessary
        if self.showCursor then
            self.cursor:setPosition(Mouse.getPosition())

            -- Make sure focus changes too
            self:mouseMoved(Mouse.getPosition())
        end

        -- Notify that the form has been shown
        if isFunction(self.formShown) then
            self:formShown()
        end
    end

    if not isNil(FormLayer.defaultInput) then
        FormLayer.defaultInput:setupEventHandlers(formLayer)
    end

    form:setLayer(formLayer)
    form:focused()

    return formLayer
end

function FormLayer.setDefaultInput(input)
    FormLayer.defaultInput = input
end

