-- Copyright (C) 2012 Jared Krinke.
-- 
-- This is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Label = { }

local setXByAlignment = {
    left = function(self, x, y) self.x = x end,
    center = function(self, x, y) self.x = x + self:getSize() / 2 end,
    right = function(self, x, y) self.x = x + self:getSize() end,
}

function Label.new(text, alignment, stretch)
    local label = Entity.new()

    label.stretch = stretch
    label.totalWidth = 0

    if not isString(text) then
        text = ""
    end

    label.textElement = label.elements:add(Element.Text.new(text))

    if isString(alignment) then
        label.alignment = alignment
        label.textElement.alignment = alignment
    end

    function label.setText(self, text)
        label.textElement.text = text
    end

    function label.setColor(self, color)
        label.textElement.color = color
    end

    function label.setLayer(self, layer)
        layer:addChild(self)
    end

    function label.getPosition(self)
        return self.x, self.y + self.height
    end

    function label.setPosition(self, x, y)
        local setXInternal = setXByAlignment[self.alignment]

        if not isFunction(setXInternal) then
            setXInternal = setXByAlignment.left
        end

        setXInternal(self, x, y)
        self.y = y - self.height
    end

    function label.setTextSize(self, width, height)
        self.width = width
        self.height = height
    end

    function label.getActive(self)
        return false
    end

    function label.getMinimumSize(self)
        return #self.textElement.text * self.width, self.height
    end

    function label.getDesiredSize(self)
        return self:getMinimumSize()
    end

    function label.getSize(self)
        local minimumWidth, minimumHeight = self:getMinimumSize()
        return self.totalWidth, minimumHeight
    end

    function label.setSize(self, width, height)
        -- Only change size if label should stretch text
        if isBoolean(self.stretch) and self.stretch then
            self.width = width / #self.textElement.text
            self.height = height
        else
            if isString(self.alignment) and self.alignment == "right" then
                -- Add padding
                local extraPadding = width - self.totalWidth

                if isNumber(self.x) then
                    self.x = self.x + extraPadding
                end
            end

            self.totalWidth = width
        end
    end

    return label
end

