-- Copyright (C) 2012 Jared Krinke.
-- 
-- This is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

local function createNode(value)
    return { value = value }
end

LinkedList = { }

function LinkedList.new()
    local linkedList = { }

    function linkedList.getLast(self)
        local last = self.head
    
        if not isNil(last) then
            while not isNil(last.next) do
                last = last.next
            end
        else
            last = nil
        end
    
        return last
    end
    
    function linkedList.append(self, value)
        local newNode = createNode(value)
        local tail = self:getLast()
    
        if not isNil(tail) then
            tail.next = newNode
            newNode.previous = tail
        else
            self.head = newNode
        end
    
        return newNode
    end
    
    function linkedList.getNext(self, node)
        return node.next
    end
    
    function linkedList.getPrevious(self, node)
        return node.previous
    end

    function linkedList.forEach(self, f)
        local node = self.head
    
        while not isNil(node) do
            f(node.value)

            node = node.next
        end
    end

    return linkedList
end

