-- Copyright (C) 2012 Jared Krinke.
-- 
-- This is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("Settings/Misc.lua")

local typeTests = {
    string = isString,
    number = isNumber,
    boolean = isBoolean,
}

MiscSettings = { }

function MiscSettings.get(t, name, default)
    local value = default

    if isTable(miscSettings) and isTable(miscSettings[t]) then
        if isFunction(typeTests[t]) and typeTests[t](miscSettings[t][name]) then
            value = miscSettings[t][name]
        end
    end

    return value
end

function MiscSettings.set(t, name, value)
    if isFunction(typeTests[t]) and typeTests[t](value) then
        -- Ensure settings entry exists
        if not isTable(miscSettings) then
            miscSettings = { }
        end

        if not isTable(miscSettings[t]) then
            miscSettings[t] = { }
        end

        -- Set the value
        miscSettings[t][name] = value
    end
end

function MiscSettings.save()
    File.createDirectory("Settings")

    local file = File.openWrite("Settings/Misc.lua")
    local i = 1

    if isTable(miscSettings) then
        file:write("miscSettings = {")

        -- Strings
        file:write("    string = {")
        
        if isTable(miscSettings.string) then
            Table.forEach(miscSettings.string, function(key, value)
                if isString(value) then
                    file:write(String.format("        %s = %q,", key, value))
                end
            end)
        end
        
        file:write("    },")

        -- Numbers
        file:write("    number = {")
        
        if isTable(miscSettings.number) then
            Table.forEach(miscSettings.number, function(key, value)
                if isNumber(value) then
                    file:write("        " .. key .. ' = ' .. value .. ',')
                end
            end)
        end
        
        file:write("    },")

        -- Booleans
        file:write("    boolean = {")
        
        if isTable(miscSettings.boolean) then
            Table.forEach(miscSettings.boolean, function(key, value)
                if isBoolean(value) then
                    if value then
                        file:write("        " .. key .. ' = true,')
                    else
                        file:write("        " .. key .. ' = false,')
                    end
                end
            end)
        end
        
        file:write("    },")

        file:write("}")
    end

    file:close()
end

