-- Copyright (C) 2012 Jared Krinke.
-- 
-- This is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("Utility/ForEach.lua")

local function createNode(value)
    return { value = value }
end

SortedList = { }

function SortedList.new(compare, ...)
    local sortedList = { }

    sortedList.compare = compare

    function sortedList.insert(self, value)
        local nextNode = self.head
        local previousNode = nil
        local node = createNode(value)

        -- Insert into sorted position
        while not isNil(nextNode) and self.compare(nextNode.value, value) <= 0 do
            previousNode = nextNode
            nextNode = nextNode.next
        end

        -- Adjust nodes
        if not isNil(previousNode) then
            previousNode.next = node
        else
            self.head = node
        end

        if not isNil(nextNode) then
            node.next = nextNode
        end
    end

    function sortedList.forEach(self, f)
        local node = self.head

        while not isNil(node) do
            f(node.value)
            node = node.next
        end
    end

    function sortedList.getFirst(self)
        local node = self.head
        local firstValue = nil

        if not isNil(node) then
            firstValue = node.value
        end

        return firstValue
    end

    function sortedList.getLast(self)
        local last = nil

        self:forEach(function(v)
            last = v
        end)

        return last
    end

    function sortedList.remove(self, value)
        local node = self.head
        local previousNode = nil

        while not isNil(node) do
            if node.value == value then
                if not isNil(previousNode) then
                    previousNode.next = node.next
                else
                    self.head = node.next
                end

                break
            end

            previousNode = node
            node = node.next
        end
    end

    function sortedList.clear(self)
        self.head = nil
    end

    function sortedList.truncate(self, max)
        if max > 0 then
            local i = 0
            local node = self.head

            while not isNil(node) do
                i = i + 1

                if i >= max then
                    node.next = nil
                    break
                end

                node = node.next
            end
        else
            self:clear()
        end
    end

    forEach(function(value) sortedList:insert(value) end, ...)

    return sortedList
end

